@@ -25,6 +25,17 @@ from magnum.common import utils
 from magnum.i18n import _
 
 
+class DNSListType(wtypes.UserType):
+    """A comman delimited dns nameserver list"""
+
+    basetype = str
+    name = "dnslist"
+
+    @staticmethod
+    def validate(value):
+        return utils.validate_dns(value)
+
+
 class MacAddressType(wtypes.UserType):
     """A simple MAC address type."""
 
@@ -129,6 +140,7 @@ class MultiType(wtypes.UserType):
                 % {'type': self.types, 'value': type(value)})
 
 
+dns_list = DNSListType()
 macaddress = MacAddressType()
 uuid = UuidType()
 name = NameType()
@@ -139,9 +151,9 @@ boolean = BooleanType()
 class JsonPatchType(wtypes.Base):
     """A complex type that represents a single json-patch operation."""
 
-    path = wtypes.wsattr(wtypes.StringType(pattern='^(/[\w-]+)+$'),
+    path = wtypes.wsattr(wtypes.StringType(pattern=r'^(/[\w-]+)+$'),
                          mandatory=True)
-    op = wtypes.wsattr(wtypes.Enum(str, 'add', 'replace', 'remove'),
+    op = wtypes.wsattr(wtypes.Enum(wtypes.text, 'add', 'replace', 'remove'),
                        mandatory=True)
     value = MultiType(wtypes.text, int)
 
@@ -168,7 +180,7 @@ class JsonPatchType(wtypes.Base):
                 '/uuid', '/project_id', '/user_id']
 
     @classmethod
-    def non_removable_attrs(self):
+    def non_removable_attrs(cls):
         """Returns a set of names of attributes that may not be removed.
 
         Attributes whose 'mandatory' property is True are automatically added
@@ -176,15 +188,15 @@ class JsonPatchType(wtypes.Base):
         field _extra_non_removable_attrs in subclasses, with a set of the form
         {'/foo', '/bar'}.
         """
-        if self._non_removable_attrs is None:
-            self._non_removable_attrs = self._extra_non_removable_attrs.copy()
-            if self._api_base:
-                fields = inspect.getmembers(self._api_base,
+        if cls._non_removable_attrs is None:
+            cls._non_removable_attrs = cls._extra_non_removable_attrs.copy()
+            if cls._api_base:
+                fields = inspect.getmembers(cls._api_base,
                                             lambda a: not inspect.isroutine(a))
                 for name, field in fields:
                     if getattr(field, 'mandatory', False):
-                        self._non_removable_attrs.add('/%s' % name)
-        return self._non_removable_attrs
+                        cls._non_removable_attrs.add('/%s' % name)
+        return cls._non_removable_attrs
 
     @staticmethod
     def validate(patch):
@@ -197,11 +209,11 @@ class JsonPatchType(wtypes.Base):
             raise wsme.exc.ClientSideError(msg % patch.path)
 
         if patch.op != 'remove':
-            if not patch.value:
+            if patch.value is None or patch.value == wtypes.Unset:
                 msg = _("'add' and 'replace' operations needs value")
                 raise wsme.exc.ClientSideError(msg)
 
         ret = {'path': patch.path, 'op': patch.op}
-        if patch.value:
+        if patch.value is not None and patch.value != wtypes.Unset:
             ret['value'] = patch.value
         return ret
